import { BinaryTreeNode } from 'src/tree';
import { CompareFn } from '..';
import { Heap } from './heap';
/**
 *
 */
export declare class SkewHeap<T> implements Heap<T> {
    /**
     * The function to determine the order of elements.
     */
    protected compare: CompareFn<T>;
    /**
     * The number of elements in the list.
     */
    protected length: number;
    /**
     * The node at the "top" of the heap.
     */
    protected root: BinaryTreeNode<T> | undefined;
    /**
     * Instantiate a heap.
     *
     * @param compareFn - The function to determine the order of elements.
     * @param elements - A set of elements to initialize the list with.
     */
    constructor(compareFn: CompareFn<T>, elements?: Iterable<T>);
    addAll(elements: Iterable<T>): number;
    clear(): void;
    comparator(): CompareFn<T>;
    contains(element: T): boolean;
    delete(element: T): boolean;
    dump(): Iterable<T>;
    merge(heap: Heap<T>): this;
    peek(): T | undefined;
    pop(): T | undefined;
    push(value: T): number;
    pushPop(value: T): T;
    replace(value: T): T | undefined;
    get size(): number;
    /**
     * Iterate through the heap in sorted order.
     *
     * **Note:** Unexpected behavior can occur if the collection is modified during iteration.
     *
     * @returns An iterator through the heap.
     */
    [Symbol.iterator](): Iterator<T>;
    update(curElement: T, newElement: T): boolean;
}
