"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.preOrderTraverse = exports.postOrderTraverse = exports.inOrderTraverse = exports.toBinaryTree = exports.clone = void 0;
const stack_1 = require("src/stack");
/**
 *
 * @param node
 *
 * @returns
 *
 * @internal
 */
function clone(node) {
    if (node == null) {
        return undefined;
    }
    const out = Object.assign({}, node);
    const stack = new stack_1.LinkedStack([out]);
    do {
        node = stack.pop();
        if (node.left) {
            stack.push((node.left = Object.assign({}, node.left)));
        }
        else if (node.right) {
            stack.push((node.right = Object.assign({}, node.right)));
        }
    } while (stack.size > 0);
    return out;
}
exports.clone = clone;
/**
 *
 * @param elements
 *
 * @returns
 *
 * @internal
 */
function toBinaryTree(elements) {
    if (elements == null || elements.length < 1) {
        return undefined;
    }
    const n = elements.length;
    const nodes = new Array(n);
    nodes[0] = { value: elements[0] };
    for (let i = 1; i < n; ++i) {
        const par = nodes[(i - 1) >>> 1];
        const node = { value: elements[i] };
        nodes[i] = node;
        if (i & 1) {
            par.left = node;
        }
        else {
            par.right = node;
        }
    }
    return nodes[0];
}
exports.toBinaryTree = toBinaryTree;
/**
 *
 * @param node
 *
 * @returns
 *
 * @internal
 */
function* inOrderTraverse(node) {
    const stack = new stack_1.LinkedStack();
    while (node != null) {
        stack.push(node);
        node = node.left;
    }
    while (stack.size > 0) {
        node = stack.pop();
        yield node;
        node = node.right;
        while (node != null) {
            stack.push(node);
            node = node.left;
        }
    }
}
exports.inOrderTraverse = inOrderTraverse;
/**
 *
 * @param node
 *
 * @returns
 *
 * @internal
 */
function* postOrderTraverse(node) {
    const stack = new stack_1.LinkedStack([{ seen: false, node }]);
    do {
        const meta = stack.pop();
        if (meta.node != null) {
            if (meta.seen) {
                yield meta.node;
            }
            else {
                meta.seen = true;
                stack.push(meta);
                stack.push({ seen: false, node: meta.node.right });
                stack.push({ seen: false, node: meta.node.left });
            }
        }
    } while (stack.size > 0);
}
exports.postOrderTraverse = postOrderTraverse;
/**
 *
 * @param node
 *
 * @returns
 *
 * @internal
 */
function* preOrderTraverse(node) {
    const stack = new stack_1.LinkedStack([node]);
    do {
        node = stack.pop();
        if (node != null) {
            yield node;
            stack.push(node.right);
            stack.push(node.left);
        }
    } while (stack.size > 0);
}
exports.preOrderTraverse = preOrderTraverse;
//# sourceMappingURL=utils.js.map